<?php

App::uses('EvSlideshowAppController', 'EvSlideshow.Controller');

/**
 * SlideshowSlides Controller
 *
 */
class SlidesController extends EvSlideshowAppController {

	public $maxDepth = 1;

/**
 * @return void
 * @see Controller::beforeFilter()
 */
	public function beforeFilter() {
		parent::beforeFilter();

		$this->Auth->allow(array(
			'fetch_slideshow'
		));

		return;
	}

/**
 * Used by the SlideshowHelper's requestAction to retrieve slides.
 *
 * @return array
 */
	public function fetch_slideshow() {
		$Model = $this->{$this->modelClass};

		return $Model->getSlides();
	}

/**
 * Setup the admin form fields.
 *
 * @return array
 */
	protected function _adminFormFields() {
		$Model = $this->{$this->modelClass};
		$modelAlias = $Model->alias;

		$fields = parent::_adminFormFields();

		if (isset($fields[$modelAlias . '.extra'])) {
			unset($fields[$modelAlias . '.extra']);
		}

		$configFields = Configure::read('EvSlideshow.fields');

		foreach ($configFields as $key => $value) {
			if ($key == 'extra') {

				foreach ($value as $extraColumn => &$extraValue) {
					if (isset($extraValue['validate'])) {
						unset($extraValue['validate']);
					}

					$extraFields['extra_' . $extraColumn] = $extraValue;
				}

				$fields = ArrayUtil::addAfter($fields, $modelAlias . '.button_link', $extraFields);
			}
		}

		return $fields;
	}

	protected function _processValidation() {
		$Model = $this->{$this->modelClass};

		// Add the custom validation if set
		$configFields = Configure::read('EvSlideshow.fields');

		if (empty($configFields)) {
			throw new exception("Slide show fields config is empty");
		}

		foreach ($configFields as $key => $value) {

			if ($key == 'Slide.extra') {

				foreach ($value as $extraKey => $extraValue) {
					if (!empty($extraValue['validate'])) {

						$Model->validate['extra_' . $extraKey] = array();

						foreach ($extraValue['validate'] as $rule => $ruleData) {
							$Model->validate['extra_' . $extraKey][$rule] = $ruleData;
						}
					}
				}

			} else {

				if (!empty($value['validate'])) {

					$column = preg_replace('|\w+\.|i', '', $key);

					foreach ($value['validate'] as $rule => $ruleData) {
						$Model->validate[$column][$rule] = $ruleData;

					}
				}

			}

		}

		return;
	}

	public function admin_edit($id = null) {
		$this->_processValidation();

		return parent::admin_edit($id);
	}

	/**
	 * Add a new image slot via AJAX.
	 *
	 * @param string $model The model to link the image to e.g. Page
	 * @param integer $key Image slot number
	 * @param string $type
	 * @param string $prefix Used when embedding images multiple levels deep e.g. Hotel.Room.0.Image
	 * @param string $plugin
	 * @param string $fields     Comma separated list of fields
	 */
	public function ajax_add($rowKey = 0) {
		$Model = $this->{$this->modelClass};
		$modelAlias = $Model->alias;

		//Set product options
		$this->set('slideKey', $rowKey);

		$this->set('fields', $this->_assembleAdminSlideFields());

		$this->set('imageSlots', $Model->imageSlots['main']);

		// assign slide populate fields
		$this->_populateSlideLookups();

		$this->set('actualModel', $modelAlias);

		$this->set('plugins', App::objects('plugin'));

		$this->theme = "Admin";

		$this->render('EvSlideshow.Slides/ajax_add', 'ajax');

		return;
	}

	/**
	 * Removes a slide
	 *
	 * @param int $id slide to be removed
	 */
	public function ajax_remove($id) {
		// Model name
		$Model = $this->{$this->modelClass};

		$this->render(false, false);

		return $Model->delete($id);
	}

	/**
	 * Change the sequence of a slide.
	 *
	 * @param int $id file to be reordered
	 * @param string $direction direction to reorder, up or down
	 */
	public function ajax_reorder($id, $direction = 'down') {
		// Model name
		$Model = $this->{$this->modelClass};

		$this->set('model', $Model->alias);

		// Fetch the currenct document
		$currentSlide = $Model->findById($id);

		if (!empty($currentSlide)) {

			// Find the relevant adjacent document
			if ($direction == 'down') {
				$order = "sequence ASC";
			} else {
				$order = "sequence DESC";
			}

			$direction = $direction == 'down' ? '>' : '<';

			$params = array(
				'conditions' => array(
					'id <>' => $id,
					"sequence $direction" => $currentSlide[$Model->alias]['sequence']
				),
				'order' => $order
			);

			$adjacentSlide = $Model->find('first', $params);

			if (! empty($adjacentSlide)) {
				// Swap the sequence numbers of the current and adjacent documents

				$data = array(
					array(
						'id' => $id,
						'sequence' => $adjacentSlide[$Model->alias]['sequence']
					),
					array(
						'id' => $adjacentSlide[$Model->alias]['id'],
						'sequence' => $currentSlide[$Model->alias]['sequence']
					)
				);

				$Model->saveMany($data);
			}

			$this->set(compact('currentSlide', 'adjacentSlide'));
		}

		$this->theme = "Admin";

		$this->render('EvSlideshow.Slides/ajax_reorder', 'ajax');

		return;
	}

}
